@@ -1,11 +1,11 @@
 package Net::Amazon::S3::Client::Object;
 {
-  $Net::Amazon::S3::Client::Object::VERSION = '0.58';
+  $Net::Amazon::S3::Client::Object::VERSION = '0.60';
 }
 use Moose 0.85;
 use MooseX::StrictConstructor 0.16;
 use DateTime::Format::HTTP;
-use Digest::MD5 qw(md5 md5_hex);
+use Digest::MD5 qw(md5_hex);
 use Digest::MD5::File qw(file_md5 file_md5_hex);
 use File::stat;
 use MIME::Base64;
@@ -16,7 +16,10 @@ use IO::File 1.14;
 # ABSTRACT: An easy-to-use Amazon S3 client object
 
 enum 'AclShort' =>
-    qw(private public-read public-read-write authenticated-read);
+    [ qw(private public-read public-read-write authenticated-read) ];
+
+enum 'StorageClass' =>
+    qw(standard reduced_redundancy);
 
 has 'client' =>
     ( is => 'ro', isa => 'Net::Amazon::S3::Client', required => 1 );
@@ -46,6 +49,18 @@ has 'content_encoding' => (
     isa      => 'Str',
     required => 0,
 );
+has 'storage_class' => (
+    is       => 'ro',
+    isa      => 'StorageClass',
+    required => 0,
+    default  => 'standard',
+);
+has 'user_metadata' => (
+    is       => 'ro',
+    isa      => 'HashRef',
+    required => 0,
+    default  => sub { {} },
+);
 
 __PACKAGE__->meta->make_immutable;
 
@@ -63,7 +78,7 @@ sub exists {
     return $http_response->code == 200 ? 1 : 0;
 }
 
-sub get {
+sub _get {
     my $self = shift;
 
     my $http_request = Net::Amazon::S3::Request::GetObject->new(
@@ -75,13 +90,30 @@ sub get {
 
     my $http_response = $self->client->_send_request($http_request);
     my $content       = $http_response->content;
+    $self->_load_user_metadata($http_response);
 
     my $md5_hex = md5_hex($content);
     my $etag = $self->etag || $self->_etag($http_response);
     confess 'Corrupted download'
       if( !$self->_is_multipart_etag($etag) && $etag ne $md5_hex);
 
-    return $content;
+    if ( $self->etag ) {
+        confess 'Corrupted download' if $self->etag ne $md5_hex;
+    } else {
+        confess 'Corrupted download'
+            if $self->_etag($http_response) ne $md5_hex;
+    }
+    return $http_response;
+}
+
+sub get {
+    my $self = shift;
+    return $self->_get->content;
+}
+
+sub get_decoded {
+    my $self = shift;
+    return $self->_get->decoded_content(@_);
 }
 
 sub get_filename {
@@ -97,6 +129,8 @@ sub get_filename {
     my $http_response
         = $self->client->_send_request($http_request, $filename);
 
+    $self->_load_user_metadata($http_response);
+
     my $md5_hex = file_md5_hex($filename);
 
     my $etag = $self->etag || $self->_etag($http_response);
@@ -104,60 +138,28 @@ sub get_filename {
       'Corrupted download' if(!$self->_is_multipart_etag($etag) && $etag ne $md5_hex);
 }
 
-sub put {
-    my ( $self, $value ) = @_;
-    my $md5        = md5($value);
-    my $md5_hex    = unpack( 'H*', $md5 );
-    my $md5_base64 = encode_base64($md5);
-    chomp $md5_base64;
+sub _load_user_metadata {
+    my ( $self, $http_response ) = @_;
 
-    my $conf = {
-        'Content-MD5'    => $md5_base64,
-        'Content-Length' => length $value,
-        'Content-Type'   => $self->content_type,
-    };
-
-    if ( $self->expires ) {
-        $conf->{Expires}
-            = DateTime::Format::HTTP->format_datetime( $self->expires );
-    }
-    if ( $self->content_encoding ) {
-        $conf->{'Content-Encoding'} = $self->content_encoding;
-    }
-    if ( $self->content_disposition ) {
-        $conf->{'Content-Disposition'} = $self->content_disposition;
+    my %user_metadata;
+    for my $header_name ($http_response->header_field_names) {
+        my ($metadata_name) = lc($header_name) =~ /\A x-amz-meta- (.*) \z/xms
+            or next;
+        $user_metadata{$metadata_name} = $http_response->header($header_name);
     }
 
-    my $http_request = Net::Amazon::S3::Request::PutObject->new(
-        s3        => $self->client->s3,
-        bucket    => $self->bucket->name,
-        key       => $self->key,
-        value     => $value,
-        headers   => $conf,
-        acl_short => $self->acl_short,
-    )->http_request;
-
-    my $http_response = $self->client->_send_request($http_request);
-
-    confess 'Error uploading' if $http_response->code != 200;
-
-    my $etag = $self->_etag($http_response);
-
-    confess 'Corrupted upload' if $etag ne $md5_hex;
+    %{ $self->user_metadata } = %user_metadata;
 }
 
-sub put_filename {
-    my ( $self, $filename ) = @_;
+sub put {
+    my ( $self, $value ) = @_;
+    $self->_put( $value, length $value, md5_hex($value) );
+}
 
-    my $md5_hex = $self->etag || file_md5_hex($filename);
-    my $size = $self->size;
-    unless ($size) {
-        my $stat = stat($filename) || confess("No $filename: $!");
-        $size = $stat->size;
-    }
+sub _put {
+    my ( $self, $value, $size, $md5_hex ) = @_;
 
-    my $md5 = pack( 'H*', $md5_hex );
-    my $md5_base64 = encode_base64($md5);
+    my $md5_base64 = encode_base64( pack( 'H*', $md5_hex ) );
     chomp $md5_base64;
 
     my $conf = {
@@ -176,23 +178,42 @@ sub put_filename {
     if ( $self->content_disposition ) {
         $conf->{'Content-Disposition'} = $self->content_disposition;
     }
-
+    if ( $self->storage_class && $self->storage_class ne 'standard' ) {
+        $conf->{'x-amz-storage-class'} = uc $self->storage_class;
+    }
+    $conf->{"x-amz-meta-\L$_"} = $self->user_metadata->{$_}
+        for keys %{ $self->user_metadata };
 
     my $http_request = Net::Amazon::S3::Request::PutObject->new(
         s3        => $self->client->s3,
         bucket    => $self->bucket->name,
         key       => $self->key,
-        value     => $self->_content_sub($filename),
+        value     => $value,
         headers   => $conf,
         acl_short => $self->acl_short,
     )->http_request;
 
     my $http_response = $self->client->_send_request($http_request);
 
-    confess 'Error uploading' . $http_response->as_string
+    confess 'Error uploading ' . $http_response->as_string
         if $http_response->code != 200;
 
-    confess 'Corrupted upload' if $self->_etag($http_response) ne $md5_hex;
+    my $etag = $self->_etag($http_response);
+
+    confess 'Corrupted upload' if $etag ne $md5_hex;
+}
+
+sub put_filename {
+    my ( $self, $filename ) = @_;
+
+    my $md5_hex = $self->etag || file_md5_hex($filename);
+    my $size = $self->size;
+    unless ($size) {
+        my $stat = stat($filename) || confess("No $filename: $!");
+        $size = $stat->size;
+    }
+
+    $self->_put( $self->_content_sub($filename), $size, $md5_hex );
 }
 
 sub delete {
@@ -351,7 +372,7 @@ Net::Amazon::S3::Client::Object - An easy-to-use Amazon S3 client object
 
 =head1 VERSION
 
-version 0.58
+version 0.60
 
 =head1 SYNOPSIS
 
@@ -448,6 +469,12 @@ This module represents objects in buckets.
   # to get the vaue of an object
   my $value = $object->get;
 
+=head2 get_decoded
+
+  # get the value of an object, and decode any Content-Encoding and/or
+  # charset; see decoded_content in HTTP::Response
+  my $value = $object->get_decoded;
+
 =head2 get_filename
 
   # download the value of the object into a file
@@ -474,8 +501,11 @@ This module represents objects in buckets.
   );
   $object->put('this is the public value');
 
-You may also set Content-Encoding using content_encoding, and
-Content-Disposition using content_disposition.
+You may also set Content-Encoding using C<content_encoding>, and
+Content-Disposition using C<content_disposition>.
+
+You may specify the S3 storage class by setting C<storage_class> to either
+C<standard> or C<reduced_redundancy>; the default is C<standard>.
 
 =head2 put_filename
 
@@ -495,8 +525,14 @@ Content-Disposition using content_disposition.
   );
   $object->put_filename('hat.jpg');
 
-You may also set Content-Encoding using content_encoding, and
-Content-Disposition using content_disposition.
+You may also set Content-Encoding using C<content_encoding>, and
+Content-Disposition using C<content_disposition>.
+
+You may specify the S3 storage class by setting C<storage_class> to either
+C<standard> or C<reduced_redundancy>; the default is C<standard>.
+
+User metadata may be set by providing a non-empty hashref as
+C<user_metadata>.
 
 =head2 query_string_authentication_uri
 
@@ -551,13 +587,26 @@ Content-Disposition using content_disposition.
   The etag and part_numbers parameters are ordered lists specifying the part
   numbers and ETags for each individual part of the multipart upload.
 
+=head2 user_metadata
+
+  my $object = $bucket->object(key => $key);
+  my $content = $object->get; # or use $object->get_filename($filename)
+
+  # return the user metadata downloaded, as a hashref
+  my $user_metadata = $object->user_metadata;
+
+To upload an object with user metadata, set C<user_metadata> at construction
+time to a hashref, with no C<x-amz-meta-> prefixes on the key names.  When
+downloading an object, the C<get>, C<get_decoded> and C<get_filename>
+methods set the contents of C<user_metadata> to the same format.
+
 =head1 AUTHOR
 
 Pedro Figueiredo <me@pedrofigueiredo.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 by Amazon Digital Services, Leon Brocard, Brad Fitzpatrick, Pedro Figueiredo.
+This software is copyright (c) 2014 by Amazon Digital Services, Leon Brocard, Brad Fitzpatrick, Pedro Figueiredo.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
