@@ -1,6 +1,6 @@
 package Net::Amazon::S3::Request;
 {
-  $Net::Amazon::S3::Request::VERSION = '0.59';
+  $Net::Amazon::S3::Request::VERSION = '0.60';
 }
 use Moose 0.85;
 use MooseX::StrictConstructor 0.16;
@@ -10,8 +10,8 @@ use Regexp::Common qw /net/;
 # ABSTRACT: Base class for request objects
 
 enum 'AclShort' =>
-    qw(private public-read public-read-write authenticated-read);
-enum 'LocationConstraint' => ( 'US', 'EU' );
+    [ qw(private public-read public-read-write authenticated-read) ];
+enum 'LocationConstraint' => [ 'US', 'EU' ];
 
 # To comply with Amazon S3 requirements, bucket names must:
 # Contain lowercase letters, numbers, periods (.), underscores (_), and dashes (-)
@@ -66,7 +66,7 @@ Net::Amazon::S3::Request - Base class for request objects
 
 =head1 VERSION
 
-version 0.59
+version 0.60
 
 =head1 SYNOPSIS
 
@@ -83,7 +83,7 @@ Pedro Figueiredo <me@pedrofigueiredo.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Amazon Digital Services, Leon Brocard, Brad Fitzpatrick, Pedro Figueiredo.
+This software is copyright (c) 2014 by Amazon Digital Services, Leon Brocard, Brad Fitzpatrick, Pedro Figueiredo.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
